; Megarom loader for MSXRAM (Multi-MegaRam cartridge)
; ver 2.07 by Jipe

; I/O port to select the ROM mapper is 8EH

; ROM mapper register format:
; 55H 16K ASCII mapper (read/Write)
; 95H 8K ASCII mapper (read/Write)
; 96H 8K SCC mapper (read/Write)
; 99H 8k Konami mapper (read/Write)

; 65H 16K ASCII mapper (read only)
; A5H 8K ASCII mapper (read only)
; A6H 8K SCC mapper (read only)
; A9H 8K Konami mapper (read only)

; ROM mapper register is ? by default

	ORG	00100H

CR	EQU	0Dh		;CR code
LF	EQU	0Ah		;LF code

BDOS	EQU	00005H		;DOS functions call 
WRSLT	EQU	00014H		;Write Slot
ENASLT	EQU	00024H		;Enable Slot
CALLF	EQU	00030H		;Call Far appel inter-slot
INITXT	EQU	0006CH		;Initialize text mode
RAMAD1	EQU	0F342H		;Main-RAM slot ID on bank 1 (4000h~7FFFH)
SETTXT	EQU	0F3B3H		;Adresse table noms SCREEN 0
EXPTBL	EQU	0FCC1H		;Main Rom emplacement du slot

MSXRAM	EQU	0006AH		;Address to check MSXRAM?

START   JP	RunPRG

Info_txt:
	DB	CR,"MSXRAM loader ver 2.07  (c) Jipe 2016",CR,LF
	DB	"MSXRAM.COM",CR,LF,CR,LF,24h

	DB	0,0,0,0,0,0,0
	DB	CR

Usage_txt:
	DB	"Usage:MSXRAM filename.ext /Sxx",CR,LF
	DB	"xx:msxram slot",CR,LF
	DB	"ext:A16 AS8 KO8 KS8 R40 R48 R80 R88",024h,CR,LF
	DB	01AH
Reset_txt:
	DB	"Press RESET Button!!",024h
ReadErr_txt:
	DB	CR,LF,"File reading error! ",024h
LoadOK_txt:
	DB	"Load complete. Thank you.",024h

A01DF:
	LD	DE,Usage_txt
	LD	HL,00080H
	LD	B,(HL)
	INC	B
	DEC	B
	JP	Z,Back2DOS
	LD	A,02FH
A01ED:
	INC	HL
	CP	(HL)
	JR	Z,A01F6
	DJNZ	A01ED
	JP	Back2DOS
A01F6:
	INC	HL
	LD	A,(HL)
	AND	0DFH
	CP	053H
	JP	NZ,Back2DOS
	CALL	A03BA
	LD	A,E
	LD	(MSXRAM_slt),A		;Store MSXRAM slot

	LD	BC,00018H
	LD	DE,00069H
	LD	HL,00068H
	LD	(HL),B
	LDIR

	LD	C,00FH		;Open File
	LD	DE,0005CH
	CALL	BDOS

	LD	HL,1
	LD	(MSXRAM),HL
	OR	A
	LD	DE,ReadErr_txt	;File Reading Error Text
	JP	NZ,Back2DOS

	LD	C,01AH		;Set disk transfer address
	LD	DE,08000H	;Required disk transfer address
	CALL	BDOS

	LD	A,(MSXRAM_slt)
	LD	H,040H
	CALL	ENASLT		;Select MSXRAM on bank 1 (4000h~7FFFH)

	XOR	A
	LD	DE,Reset_txt	;Improper Cartridge Text
	LD	BC,04000H
	LD	HL,04000H
	LD	A,(HL)
	CPL
	LD	(HL),A
	CP	(HL)
	CPL
	LD	(HL),A
	JP	Z,Back2DOS		;Jump if MSXRAM not found

	XOR	A
	CALL	RO_Mode_sel 	;OUT &H8E,&H65

	LD	(06800H),A
	LD	(BC),A
	CALL	Wrt_to_6000 	;LD (&h6000),A

	LD	(07000H),A
	LD	A,080H
	LD	(BC),A
	LD	A,010H
	CALL	RW_Mode_sel 	;OUT &H8E,&H55

	LD	A,(BC)
	XOR	(HL)
	JR	Z,A0266
	LD	A,1
A0266:	LD	(D03E4),A
	LD	A,(HL)
	CPL
	LD	(HL),A
	CP	(HL)
	CPL
	LD	(HL),A
	JR	Z,A0279
	XOR	A
	CALL	RO_Mode_sel 	;OUT &H8E,&H65

	LD	(BC),A
	JP	Back2DOS

A0279:
	PUSH	AF
	LD	A,(BC)
	CPL
	LD	(HL),A
	LD	A,(BC)
	CP	(HL)
	JR	Z,A0286
	LD	A,1
	LD	(D03E4),A
A0286:	POP	AF
	LD	(HL),A
	XOR	A
	CALL	RO_Mode_sel 	;OUT &H8E,&H65

	LD	(BC),A
	LD	(D03E5),A
	INC	A
	LD	(06800H),A
	LD	(07000H),A
	LD	A,(RAMAD1)
	LD	H,040H
	CALL	ENASLT		;Select Main-RAM on bank 1 (4000h~7FFFH)

	LD	A,(D03E4)
	OR	A
CallBDOS:
	DB	0,0,0  		;Replaced by CALL BDOS
A02A6:
	LD	C,027H		;Load bloc file: RANDOM BLOCK READ
	LD	DE,0005CH	;Call Dos
LDHL4000H:	
	LD	HL,04000H	;40H replaced by 20H for 8K mappers
	CALL	BDOS
	LD	A,H
	OR	L
	LD	DE,LoadOK_txt
	JP	Z,A0340		;Jump if file loaded

	LD	A,H
CP40H:
	CP	040H  		;40H replaced by 20H for 8K mappers
	LD	DE,ReadErr_txt
	JP	NZ,Back2DOS	;Jump if file error

	LD	A,(MSXRAM_slt)
	LD	H,040H
	CALL	ENASLT		;Select MSXRAM on bank 1 (4000h~7FFFH)
	XOR	A
	CALL	RO_Mode_sel 	;OUT &H8E,&H65
T02CE:
	CALL	Wrt_to_6000 	;LD (6000H),A

	LD	A,(D03E5)
	CALL	Wrt_to_6000

	NOP
	NOP
	NOP
	NOP
	NOP
	LD	A,010H
	CALL	RW_Mode_sel
LDBC04000H:
	LD	BC,04000H	;4000H replaced by 2000H for 8K mappers
	LD	DE,04000H
	LD	HL,08000H
	LDIR

	XOR	A
	CALL	RO_Mode_sel 	;OUT &H8E,&H65
	CALL	Wrt_to_6000 	;LD (6000H),A

	LD	A,(RAMAD1)
	LD	H,040H
	CALL	ENASLT		;Select Main-RAM on bank 1 (4000h~7FFFH)

	LD	A,(D03E5)
	INC	A
	LD	(D03E5),A
	DEC	A
	AND	00FH
	CP	00AH
	JR	C,A030B		;Jump if character = LF

	ADD	A,007H
A030B:	ADD	A,030H
	LD	E,A
	LD	C,002H		;Console Output
	CALL	BDOS

	LD	A,(D03E5)
	AND	00FH
	JP	NZ,A02A6

	LD	E,00DH		;CR
	LD	C,002H		;Console Output
	CALL	BDOS

	LD	E,00AH		;LF
	LD	C,002H		;Console Output
	CALL	BDOS

	JP	A02A6

Wrt_to_6000:
	LD	(06000H),A	;Replaced by 4000H for KO8 , 5000H for KS8
	RET

RW_Mode_sel:
	PUSH	AF
VA55H:
	LD	A,055H	;Replaced by 99H for KO8, 96H for KS8, 95H for KS8, 55H for A16
	OUT	(08EH),A
	POP	AF
	RET

	NOP

RO_Mode_sel:
	PUSH	AF
VA65H:
	LD	A,065H	;Replaced by A9H for KO8, A6H for KS8, A5H for KS8, 65H for A16
	OUT	(08EH),A
	POP	AF
	RET

	NOP

A0340:
	LD	A,(MSXRAM_slt)
LDHL6000H:
	LD	HL,06000H	;Replaced by 4000H for KO8, 5000H for KS8
LDE0H:
	LD	E,0		;Replaced by 0 for KO8, 0 for KS8
	CALL	WRSLT
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	A,(MSXRAM_slt)
LDHL6000H2:
	LD	HL,06000H	;Replaced by 6000H for KO8, 7000H for KS8
LDE0H2:
	LD	E,0		;Replaced by 0 for KO8, 0 for KS8
	CALL	WRSLT
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	A,(MSXRAM_slt)
LDHL7000H:
    	LD	HL,07000H	;Replaced by 8000H for KO8, 9000H for KS8
LDE1H:
	LD	E,1		;Replaced by 0 for KO8, 0 for KS8, 0 for A16
	CALL	WRSLT
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	A,(MSXRAM_slt)
LDHL6000H3:
	LD	HL,06000H	;Replaced by A000H for KO8, B000H for KS8
LDE0H3:
	LD	E,0		;Replaced by 0 for KO8, 0 for KS8
	CALL	WRSLT
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	HL,BASPRG_exe
	LD	DE,0C000H
	LD	BC,BASPRG_exe_end-BASPRG_exe	;old value 00020H
	LDIR
	JP	0C000H

	NOP
	NOP

; Execute the Basic program in ROM

BASPRG_exe:
	LD	A,(EXPTBL)
	LD	H,0
	CALL	ENASLT		;Select BIOS ROM on bank 0 (0000h~3FFFH)

	LD	A,0
	LD	(SETTXT),A
	NOP
	NOP
	NOP
	CALL	INITXT

	LD	A,(EXPTBL)
	LD	H,040H
	CALL	ENASLT		;Select BIOS ROM on bank 1 (4000h~7FFFH)
	JP	0000H		;Soft Reset
BASPRG_exe_end:

	NOP
	NOP
	NOP
	NOP
	NOP

Back2DOS:
	POP	AF
	LD	C,009H		;Print text
	CALL	BDOS
	RST	0		;Back to MSX-DOS

A03BA:
	LD	DE,0
A03BD:
	INC	HL
	LD	A,(HL)
	SUB	030H
	RET	C
	CP	00AH
	JR	C,A03CF
	AND	0DFH
	SUB	007H
	RET	C
	CP	010H
	CCF
	RET	C
A03CF:
	SLA	E
	RL	D
	SLA	E
	RL	D
	SLA	E
	RL	D
	SLA	E
	RL	D
	ADD	A,E
	LD	E,A
	JR	A03BD

MSXRAM_slt:
	DB	0      		;slot number
D03E4:
	DB	0		;Flag?
D03E5:
	DB	0      		;Page number?

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

RunPRG:	CALL	RO_Mode_sel 	;OUT&H8E,&H65 (ASCII 16 mapper)
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

; Check file extension mame

	LD	HL,00065H	;Analysis 1st character
	LD	A,(HL)
	CP	04BH  		;Compare with "K"
	NOP
	JP	NZ,A0410	;jump if not "K"
	JP	A0450

	NOP
	NOP
	NOP

A0410:	LD	HL,00066H	;Analysis 2nd character
	LD	A,(HL)
	CP	038H  		;Compare with "8"
	NOP
	JP	NZ,A0420	;jump if not "8"
	CALL	A0530
	NOP
	NOP
	NOP
A0420:	LD	HL,00066H	;Analysis 2nd character
	LD	A,(HL)
	CP	031H  		;Compare with "1"
	NOP
	JP	NZ,A0434	;jump if not "1"
	LD	A,0
	LD	(LDE1H+1),A
	NOP
	JP	A04D0

	NOP

A0434:	LD	HL,00066H	;Analysis 2nd character
	LD	A,(HL)
	CP	053H  		;Compare with "S"
	NOP
	JP	NZ,A0450	;jump if not "S"

	LD	A,095H
	LD	(VA55H+1),A
	LD	A,0A5H
	LD	(VA65H+1),A
	NOP
	NOP
	NOP
	NOP
	JP	A04D0

	NOP

A0450:	LD	HL,00066H	;Analysis 1st character
	LD	A,(HL)
	CP	04FH  		;Compare with "O"
	NOP
	JP	NZ,A0490	;jump if not "O"
	LD	A,099H
	LD	(VA55H+1),A
	LD	A,0A9H
	LD	(VA65H+1),A

	LD	A,040H
	LD	(LDHL6000H2+2),A
	LD	(Wrt_to_6000+2),A
	LD	A,060H
	LD	(LDHL6000H2+2),A
	LD	A,080H
	LD	(LDHL7000H+2),A
	LD	A,0A0H
	LD	(LDHL6000H3+2),A

	LD	A,0
	LD	(LDE0H+1),A
	INC	A
	LD	(LDE0H2+1),A
	INC	A
	LD	(LDE1H+1),A
	INC	A
	LD	(LDE0H3+1),A
	JP	A04D0

	NOP

A0490:	LD	HL,00066H	;Analysis 2nd character
	LD	A,(HL)
	CP	053H  		;Compare with "S"
	NOP
	JP	NZ,A04D0
	LD	A,096H
	LD	(VA55H+1),A
	LD	A,0A6H
	LD	(VA65H+1),A

	LD	A,050H
	LD	(LDHL6000H+2),A
	LD	(Wrt_to_6000+2),A
	LD	A,070H
	LD	(LDHL6000H2+2),A
	LD	A,090H
	LD	(LDHL7000H+2),A
	LD	A,0B0H
	LD	(LDHL6000H3+2),A

	LD	A,0
	LD	(LDE0H+1),A
	INC	A
	LD	(LDE0H2+1),A
	INC	A
	LD	(LDE1H+1),A
	INC	A
	LD	(LDE0H3+1),A
	JP	A04D0

	NOP

A04D0:	LD	HL,00067H	;Analysis 3rd character
	LD	A,(HL)
	CP	038H  		;Compare with "8"
	NOP
	JP	NZ,A04F0
	LD	A,020H
	LD	(LDHL4000H+2),A
	LD	(CP40H+1),A
	LD	(LDBC04000H+2),A
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
A04F0:
	JP	A01DF
;
; >> NO EXECUTION PATH TO HERE <<
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	A,(MSXRAM_slt)
	LD	H,040H
	CALL	ENASLT		;Select MSXRAM on bank 1 (4000h~7FFFH)
	XOR	A
	CALL	RO_Mode_sel 		;OUT&h8E,&H65
	LD	(06000H),A
	CALL	RW_Mode_sel 		;OUT&H8E,&H55
	LD	HL,04000H
	LD	DE,04001H
	LD	BC,04000H
	LD	(HL),000H
	LDIR
	XOR	A
	CALL	RO_Mode_sel 		;OUT&h8E,&H65
	RET
;
; >> NO EXECUTION PATH TO HERE <<
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
A0530:	LD	A,0CDH
	LD	(CallBDOS),A
	LD	A,000H
	LD	(CallBDOS+1),A
	LD	A,005H
	LD	(CallBDOS+2),A
	NOP
	LD	HL,T0570
	LD	DE,T02CE
	LD	BC,00010H
	LDIR
	RET
;
; >> NO EXECUTION PATH TO HERE <<
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
T0570:
	NOP
	NOP
	NOP
	LD	A,001H
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	LD	(06000H),A
	NOP
	NOP
	CALL	CALLF
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	DB	"JIPE"

	NOP

	END
